//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : CGEffect.cpp
// Description : Classe permettant de stocker les informations necessaires
//               a un effet multi-passes Cg.
//===========================================================================


#include <stdio.h>
#include "./CGEffect.h"


//-------------------------------------------------------------------------
// CONSTRUCTEURS - DESTRUCTEUR --------------------------------------------
//-------------------------------------------------------------------------

CCGEffect::CCGEffect( void )
{
   // mise a NULL de toutes les passes
   for( int i = 0 ; i < MAX_PASS ; i++ )
   {
      m_pAllPass[i] = NULL ;
   }
   
   // pas de passes au debut
   m_uiNbPass      = 0 ;
   m_uiCurrentPass = 0 ;
}

//-------------------------------------------------------------------------

CCGEffect::~CCGEffect( void )
{
   // destruction des passes
   // TODO : il ne faut pas faire ca si on veut pouvoir utiliser une meme
   // passe dans plusieurs effets. Ou alors, il faut introduire des
   // Smart Pointers qui gereront les occurences pour nous.
   for( int i = 0 ; i < m_uiNbPass ; i++ )
   {
      delete m_pAllPass[i] ;
   }
}





//-------------------------------------------------------------------------
// ACCESSEURS -------------------------------------------------------------
//-------------------------------------------------------------------------

CCGPass * CCGEffect::getPass( unsigned int num ) const
{
   if( num <= MAX_PASS ) { return m_pAllPass[num - 1] ; }

   return NULL ;
}






//-------------------------------------------------------------------------
// INTERFACE UTILISATION --------------------------------------------------
//-------------------------------------------------------------------------

CCGPass * CCGEffect::getFirstPass( void )
{
   m_uiCurrentPass = 0 ;
   return m_pAllPass[0] ;
}

//-------------------------------------------------------------------------

CCGPass * CCGEffect::getNextPass( void )
{
   m_uiCurrentPass++ ;
   if( m_uiCurrentPass >= MAX_PASS ) { return NULL ; }
   
   return m_pAllPass[m_uiCurrentPass] ;
}

//-------------------------------------------------------------------------

CCGPass * CCGEffect::getLastPass( void )
{
   m_uiCurrentPass = m_uiNbPass - 1 ;
   return m_pAllPass[m_uiCurrentPass] ;
}

//-------------------------------------------------------------------------

CCGPass * CCGEffect::getPreviousPass( void )
{
   m_uiCurrentPass-- ;
   if( m_uiCurrentPass < 0 ) { return NULL ; }
   
   return m_pAllPass[m_uiCurrentPass] ;
}






//-------------------------------------------------------------------------
// MODIFIEURS -------------------------------------------------------------
//-------------------------------------------------------------------------

bool CCGEffect::pushPass( CCGVertexShader * vs           ,
                          CCGPixelShader  * ps           ,
                          bool              zenable      ,
                          bool              zwrite       ,
                          bool              alpha        ,
                          bool              blend        ,
                          GLenum            alphaFunc    ,
                          GLclampf          alphaRef     ,
                          GLenum            blendSrcFunc ,
                          GLenum            blendDestFunc )
{
   if( m_uiNbPass == MAX_PASS ) { return false ; }

   m_pAllPass[m_uiNbPass] = new CCGPass( vs          , ps           ,
                                         zenable     , zwrite       ,
					 alpha       , blend        ,
					 alphaFunc   , alphaRef     ,
					 blendSrcFunc, blendDestFunc ) ;
   m_uiNbPass++ ;

   return true ;
}

//-------------------------------------------------------------------------

void CCGEffect::popPass( void )
{
   m_uiNbPass-- ;
   delete m_pAllPass[m_uiNbPass] ;
   m_pAllPass[m_uiNbPass] = NULL ;
}

//-------------------------------------------------------------------------
