//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : CGPass.h
// Description : Classe permettant de stocker les informations necessaires
//               a une passe pour une effet Cg.
//===========================================================================


#ifndef __CGPASS_H
#define __CGPASS_H

#include "./CGPixelShader.h"
#include "./CGVertexShader.h"


// TODO : cette classe utilisant les operations fournies par OpenGL pour
// le blending ne semble pas assez avancee pour faire tout ce que l'on
// voudrait pouvoir faire. Utiliser les Register Combiners serait peut
// etre mieux ... mais il faut les apprendre avant de pouvoir s'en servir :)


//-------------------------------------------------------------------------
class CCGPass
{

public :
   
   //----------------------------------------------------------------
   CCGPass( void ) ;
   CCGPass( CCGVertexShader * vs                                    ,
            CCGPixelShader  * ps                                    ,
	    bool              zenable       = true                  ,
            bool              zwrite        = true                  ,
	    bool              alpha         = false                 ,
	    bool              blend         = false                 ,
            GLenum            alphaFunc     = GL_GREATER            ,
	    GLclampf          alphaRef      = 0.0f                  ,
            GLenum            blendSrcFunc  = GL_SRC_ALPHA          ,
            GLenum            blendDestFunc = GL_ONE_MINUS_SRC_ALPHA ) ;
   virtual ~CCGPass( void ) ;
   // TODO : un constructeur de copie serait certainement bien utile si
   // on veut pouvoir utiliser plusieurs shaders dans differents effets
   //----------------------------------------------------------------

   //----------------------------------------------------------------
   // ACCESSEURS
   CCGVertexShader * getVertexShader( void ) const { return m_pVS ; }
   CCGPixelShader  * getPixelShader(  void ) const { return m_pPS ; }

   bool haveZEnable( void ) const { return m_bZEnable ; }
   bool haveZWrite(  void ) const { return m_bZWrite  ; }
   bool haveAlpha(   void ) const { return m_bAlpha   ; }
   bool haveBlend(   void ) const { return m_bBlend   ; }
   
   GLclampf getAlphaRef( void ) const { return m_fAlphaRef ; }

   GLenum getAlphaFunc(     void ) const { return m_eAlphaFunc     ; }
   GLenum getBlendSrcFunc(  void ) const { return m_eBlendSrcFunc  ; }
   GLenum getBlendDestFunc( void ) const { return m_eBlendDestFunc ; }
   //----------------------------------------------------------------

   //----------------------------------------------------------------
   // INTERFACE UTILISATION
   virtual void enable(  void ) ;
   virtual void disable( void ) ;
   //----------------------------------------------------------------

   //----------------------------------------------------------------
   // MODIFIEURS
   void setVertexShader( CCGVertexShader * vs ) ;
   void setPixelShader(  CCGPixelShader  * ps ) ;
   
   void setMeltingParams( bool     zenable      ,
                          bool     zwrite       ,
			  bool     alpha        ,
			  bool     blend        ,
			  GLenum   alphaFunc    ,
			  GLclampf alphaRef     ,
			  GLenum   blendSrcFunc ,
			  GLenum   blendDestFunc ) ;

   void setZEnable( bool truth ) ;
   void setZWrite(  bool truth ) ;
   void setAlpha(   bool truth ) ;
   void setBlend(   bool truth ) ;
   
   void setAlphaRef( GLclampf ref ) ;

   void setAlphaFunc(     GLenum func ) ;
   void setBlendSrcFunc(  GLenum func ) ;
   void setBlendDestFunc( GLenum func ) ;
   //----------------------------------------------------------------

protected :

   CCGVertexShader * m_pVS ;
   CCGPixelShader  * m_pPS ;

   bool     m_bZEnable       ;
   bool     m_bZWrite        ;
   bool     m_bAlpha         ;
   bool     m_bBlend         ;
   GLenum   m_eAlphaFunc     ;
   GLenum   m_eBlendSrcFunc  ;
   GLenum   m_eBlendDestFunc ;
   GLclampf m_fAlphaRef      ;

} ;
//-------------------------------------------------------------------------

#endif //__CGPASS_H
