//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : GLTexture.cpp
// Description : Classe contenant les informations d'une texture OpenGL
//===========================================================================


#include <stdio.h>
#include <qgl.h>
#include "./GLTexture.h"


//-------------------------------------------------------------------------
// CONSTRUCTEURS - DESTRUCTEUR --------------------------------------------
//-------------------------------------------------------------------------

CGLTexture::CGLTexture( const char * filename )
{
   QImage temp( filename ) ;
   m_pImage      = QGLWidget::convertToGLFormat( temp ) ;
   m_uiGLTexName = 0                                    ;
}

//-------------------------------------------------------------------------

CGLTexture::~CGLTexture()
{
   if( m_uiGLTexName != 0 ) { glDeleteTextures( 1, &m_uiGLTexName ) ; }
}





//-------------------------------------------------------------------------
// INTERFACE UTILISATION --------------------------------------------------
//-------------------------------------------------------------------------

GLuint CGLTexture::createGLTexture( GLenum wraps    ,
                                    GLenum wrapt    ,
                                    GLenum magfilter,
				    GLenum minfilter )
{
   glGenTextures( 1, &m_uiGLTexName ) ;
   if( m_pImage.height() == 1 )
   {
      glBindTexture( GL_TEXTURE_1D, m_uiGLTexName ) ;
      glTexParameteri( GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, wraps ) ;
      glTexParameteri( GL_TEXTURE_1D, GL_TEXTURE_WRAP_T, wrapt ) ;
      glTexParameteri( GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, magfilter ) ;
      glTexParameteri( GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, minfilter ) ;
      
      // TODO : il faudrait faire attention au type de stockage dans la QImage
      // pour changer le GL_RGBA. Vu que dans le projet on utilisera que des
      // textures en RGBA, on ne perd pas de temps a tout gerer.
      glTexImage1D( GL_TEXTURE_1D   ,    // texture cible
                    0               ,    // niveau mipmap
		    GL_RGBA         ,    // format interne
		    m_pImage.width(),    // largeur de la texture
		    0               ,    // taille des bords
		    GL_RGBA         ,    // format des donnees
		    GL_UNSIGNED_BYTE,    // type de chaque composante
		    m_pImage.bits()  ) ; // donnees
   }
   else
   {
      glBindTexture( GL_TEXTURE_2D, m_uiGLTexName ) ;
      glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wraps ) ;
      glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrapt ) ;
      glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, magfilter ) ;
      glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, minfilter ) ;

      // TODO : il faudrait faire attention au type de stockage dans la QImage
      // pour changer le GL_RGBA. Vu que dans le projet on utilisera que des
      // textures en RGBA, on ne perd pas de temps a tout gerer.
      glTexImage2D( GL_TEXTURE_2D    ,    // texture cible
                    0                ,    // niveau mipmap
		    GL_RGBA          ,    // format interne
                    m_pImage.width() ,    // largeur de la texture
		    m_pImage.height(),    // hauteur de la texture
		    0                ,    // taille des bords
		    GL_RGBA          ,    // format des donnees
		    GL_UNSIGNED_BYTE ,    // type de chaque composante
		    m_pImage.bits()   ) ; // donnees
   }
}

//-------------------------------------------------------------------------

GLuint CGLTexture::createGLTexName( void )
{
   glGenTextures( 1, &m_uiGLTexName ) ;
   return m_uiGLTexName ;
}

//-------------------------------------------------------------------------
