//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : GLWindow.h
// Description : Classe contenant les informations d'une fenetre
//===========================================================================


#ifndef __GLWINDOW_H
#define __GLWINDOW_H

#include <X3DTK/simplex3dglscene.h>
#include <X3DTK/simplifiedmeshscene.h>
#include <X3DTK/MESH/facesdrawer.h>
#include <QGLViewer/qglviewer.h>
#include "./GLTexture.h"
#include "./CGShaderManager.h"
#include "./CGSketchVS.h"
#include "./CGSketchPS.h"
#include "./CGEffect.h"


//-------------------------------------------------------------------------
class CGLWindow : public QGLViewer
{

public :

   //-----------------------------------------------------------------
   // CONSTRUCTEURS - DESTRUCTEUR
   CGLWindow( void ) ;
   virtual ~CGLWindow( void ) ;
   //-----------------------------------------------------------------
   
protected :
      
   //-----------------------------------------------------------------
   // ACTIONS
   virtual void draw( void ) ;
   virtual void init( void ) ;

   virtual void keyPressEvent( QKeyEvent * e ) ;

   virtual void help( void ) const ;
   //-----------------------------------------------------------------

   //-----------------------------------------------------------------
   // PARAMETRAGE
   virtual void loadSceneFile( void ) ;
   virtual void loadBrushFile( void ) ;

   virtual void changeDisplacment( void ) ;
   virtual void changeFactor(      void ) ;
   //-----------------------------------------------------------------

   //-----------------------------------------------------------------
   // ACCESSEURS
   virtual QString helpString( void ) const ;
   //-----------------------------------------------------------------


   CCGShaderManager * m_pShaderManager ;
   CCGSketchVS      * m_pSketchVS      ;
   CCGSketchPS      * m_pSketchPS      ;

   CCGEffect * m_pEffect ;

   CGLTexture * m_pBrushGLTex      ;
   CGLTexture * m_pBackgroundGLTex ;
   
   // display liste de la sky box
   GLuint m_uiSkyBoxList ;

   // parametres des shaders
   QColor m_oConstColor  ;
   float  m_fDisplacment ;
   float  m_fFactor      ;

   X3DTK::SimplifiedMeshScene m_oMeshScene ; // la scene
   X3DTK::MESH::FacesDrawer   m_oDrawer    ; // l'afficheur de scene

} ;
//-------------------------------------------------------------------------

#endif //__GLWINDOW_H
